@@ -195,12 +195,12 @@ setMethod("sim", c(x = "ph"), function(x, n = 1000) {
 #' @examples
 #' obj <- ph(structure = "general")
 #' dens(obj, c(1, 2, 3))
-setMethod("dens", c(x = "ph"), function(x, y = seq(0, quan(x, .95)$quantile, length.out = 10)) {
+setMethod("dens", c(x = "ph"), function(x, y) {
   y_inf <- (y == Inf)
   dens <- y
   dens[!y_inf] <- phdensity(y, x@pars$alpha, x@pars$S)
   dens[y_inf] <- 0
-  return(list(y = y, dens = dens))
+  return(dens)
 })
 
 #' Distribution Method for phase type distributions
@@ -216,13 +216,13 @@ setMethod("dens", c(x = "ph"), function(x, y = seq(0, quan(x, .95)$quantile, len
 #' obj <- ph(structure = "general")
 #' cdf(obj, c(1, 2, 3))
 setMethod("cdf", c(x = "ph"), function(x,
-                                       q = seq(0, quan(x, .95)$quantile, length.out = 10),
+                                       q,
                                        lower.tail = TRUE) {
   q_inf <- (q == Inf)
   cdf <- q
   cdf[!q_inf] <- phcdf(q[!q_inf], x@pars$alpha, x@pars$S, lower.tail)
   cdf[q_inf] <- as.numeric(1 * lower.tail)
-  return(list(q = q, cdf = cdf))
+  return(cdf)
 })
 
 #' Hazard rate Method for phase type distributions
@@ -236,10 +236,10 @@ setMethod("cdf", c(x = "ph"), function(x,
 #' @examples
 #' obj <- ph(structure = "general")
 #' haz(obj, c(1, 2, 3))
-setMethod("haz", c(x = "ph"), function(x, y = seq(0, quan(x, .95)$quantile, length.out = 10)) {
-  d <- dens(x, y)$dens
-  s <- cdf(x, y, lower.tail = FALSE)$cdf
-  return(list(y = y, haz = d / s))
+setMethod("haz", c(x = "ph"), function(x, y) {
+  d <- dens(x, y)
+  s <- cdf(x, y, lower.tail = FALSE)
+  return(d / s)
 })
 
 #' Quantile Method for phase type distributions
@@ -254,12 +254,12 @@ setMethod("haz", c(x = "ph"), function(x, y = seq(0, quan(x, .95)$quantile, leng
 #' obj <- ph(structure = "general")
 #' quan(obj, c(0.5, 0.9, 0.99))
 setMethod("quan", c(x = "ph"), function(x,
-                                        p = seq(0, 1, length.out = 10)) {
+                                        p) {
   quan <- numeric(length(p))
   for (i in seq_along(p)) {
-    quan[i] <- stats::uniroot(f = function(q) p[i] - cdf(x, 1 / (1 - q) - 1)$cdf, interval = c(0, 1))$root
+    quan[i] <- stats::uniroot(f = function(q) p[i] - cdf(x, 1 / (1 - q) - 1), interval = c(0, 1))$root
   }
-  return(list(p = p, quantile = 1 / (1 - quan) - 1))
+  return(1 / (1 - quan) - 1)
 })
 
 #' Fit Method for ph Class
@@ -270,12 +270,20 @@ setMethod("quan", c(x = "ph"), function(x,
 #' @param rcen vector of right-censored observations
 #' @param rcenweight vector of weights for right-censored observations.
 #' @param stepsEM number of EM steps to be performed.
+#' @param methods methods to use for matrix exponential calculation: RM, UNI or PADE
 #' @param rkstep Runge-Kutta step size (optional)
+#' @param uni_epsilon epsilon parameter for uniformization method
 #' @param maxit maximum number of iterations when optimizing g function.
 #' @param reltol relative tolerance when optimizing g function.
 #' @param every number of iterations between likelihood display updates.
+#' @param plot logical indicating whether to plot the fit at each iteration.
 #' 
 #' @return An object of class \linkS4class{ph}.
+#' 
+#' @importFrom grDevices dev.off
+#' @importFrom graphics hist legend lines
+#' @importFrom utils head
+#' 
 #' @export
 #'
 #' @examples
@@ -290,22 +298,39 @@ setMethod(
            rcen = numeric(0),
            rcenweight = numeric(0),
            stepsEM = 1000,
+           methods = c("RK", "RK"),
            rkstep = NA,
+           uni_epsilon = NA,
            maxit = 100,
            reltol = 1e-8,
-           every = 100) {
+           every = 100,
+           plot = FALSE) {
+    EMstep <- eval(parse(text = paste("EMstep_", methods[1], sep = "")))
+    if(!all(c(y, rcen) > 0)) stop("data should be positive")
+    if(!all(c(weight, rcenweight) >= 0)) stop("weights should be non-negative")
     is_iph <- methods::is(x, "iph")
-    if (is_iph) {
+    if (!is_iph) {
+      LL <- eval(parse(text = paste("logLikelihoodPH_", methods[2], sep = "")))
+    }else if (is_iph) {
       par_g <- x@gfun$pars
       inv_g <- x@gfun$inverse
-      mLL <- eval(parse(text = paste("logLikelihoodM", x@gfun$name, "_RK", sep = "")))
+      LL <- eval(parse(text = paste("logLikelihoodM", x@gfun$name, "_", methods[2], sep = "")))
     }
     A <- data_aggregation(y, weight)
     y <- A$un_obs
     weight <- A$weights
-    B <- data_aggregation(rcen, rcenweight)
-    rcen <- B$un_obs
-    rcenweight <- B$weights
+    if(length(rcen)>0){
+      B <- data_aggregation(rcen, rcenweight)
+      rcen <- B$un_obs
+      rcenweight <- B$weights
+    }
+    if(plot == TRUE){
+      if(length(rcen)>0) stop("plot option only available for non-censored data")
+      h <- hist(rep(y, weight), breaks = 30, plot = FALSE)
+      sq <- seq(0, 1.1 * max(y), length.out = 200)
+      plot(head(h$breaks, length(h$density)), h$density, col = "#b2df8a", 
+           main = "Histogram", xlab = "data", ylab = "density", type = "s", lwd = 2)
+    }
 
     ph_par <- x@pars
     alpha_fit <- clone_vector(ph_par$alpha)
@@ -313,20 +338,44 @@ setMethod(
 
     if (!is_iph) {
       for (k in 1:stepsEM) {
-        if(!is.na(rkstep)) RKstep <- rkstep else  RKstep <- default_step_length(S_fit)
-        EMstep_RK(RKstep, alpha_fit, S_fit, y, weight, rcen, rcenweight)
+        epsilon1 <- switch(which(methods[1] == c("RK", "UNI","PADE")),
+                           if(!is.na(rkstep)){rkstep} else{default_step_length(S_fit)},
+                           if(!is.na(uni_epsilon)){uni_epsilon} else{1e-4},
+                           0)
+        epsilon2 <- switch(which(methods[2] == c("RK", "UNI","PADE")),
+                           if(!is.na(rkstep)){rkstep} else{default_step_length(S_fit)},
+                           if(!is.na(uni_epsilon)){uni_epsilon} else{1e-4},
+                           0)
+        EMstep(epsilon1, alpha_fit, S_fit, y, weight, rcen, rcenweight)
         if (k %% every == 0) {
           cat("\r", "iteration:", k,
-            ", logLik:", logLikelihoodPH_RK(RKstep, alpha_fit, S_fit, y, weight, rcen, rcenweight),
+            ", logLik:", LL(epsilon2, alpha_fit, S_fit, y, weight, rcen, rcenweight),
             sep = " "
           )
+          if(plot == TRUE){
+            dev.off()
+            plot(head(h$breaks, length(h$density)), h$density, col = "#b2df8a", 
+                 main = "Histogram", xlab = "data", ylab = "density", type = "s", lwd = 2)
+            tmp_ph <- ph(alpha_fit, S_fit)
+            lines(sq, dens(tmp_ph, sq), col = "#33a02c", lwd = 2, lty = 1)
+            legend("topright", 
+                   legend = c("Data", "PH fit"), 
+                   col = c("#b2df8a", "#33a02c"), 
+                   lty = c(1,1), 
+                   bty = "n", 
+                   lwd = 2, 
+                   cex = 1.2, 
+                   text.col = "black", 
+                   horiz = FALSE, 
+                   inset = c(0.05, 0.05))
+          }
         }
       }
       cat("\n", sep = "")
       x@pars$alpha <- alpha_fit
       x@pars$S <- S_fit
       x@fit <- list(
-        logLik = logLikelihoodPH_RK(RKstep, alpha_fit, S_fit, y, weight, rcen, rcenweight),
+        logLik = LL(epsilon2, alpha_fit, S_fit, y, weight, rcen, rcenweight),
         nobs = sum(A$weights)
       )
     }
@@ -337,13 +386,20 @@ setMethod(
         if(x@gfun$name != "gev") {trans <- inv_g(par_g, y); trans_cens <- inv_g(par_g, rcen)
         }else{ t <- inv_g(par_g, y, weight); tc <- inv_g(par_g, rcen, rcenweight) 
         trans <- t$obs; trans_weight <- t$weight; trans_cens <- tc$obs; trans_rcenweight <- tc$weight}
-        if(!is.na(rkstep)) RKstep <- rkstep else  RKstep <- default_step_length(S_fit)
-        EMstep_RK(RKstep, alpha_fit, S_fit, trans, trans_weight, trans_cens, trans_rcenweight)
+        epsilon1 <- switch(which(methods[1] == c("RK", "UNI","PADE")),
+                           if(!is.na(rkstep)){rkstep} else{default_step_length(S_fit)},
+                           if(!is.na(uni_epsilon)){uni_epsilon} else{1e-4},
+                           0)
+        epsilon2 <- switch(which(methods[2] == c("RK", "UNI","PADE")),
+                           if(!is.na(rkstep)){rkstep} else{default_step_length(S_fit)},
+                           if(!is.na(uni_epsilon)){uni_epsilon} else{1e-4},
+                           0)
+        EMstep(epsilon1, alpha_fit, S_fit, trans, trans_weight, trans_cens, trans_rcenweight)
         opt <- suppressWarnings(
           stats::optim(
             par = par_g,
-            fn = mLL,
-            h = RKstep,
+            fn = LL,
+            h = epsilon2,
             alpha = alpha_fit,
             S = S_fit,
             obs = y,
@@ -353,23 +409,41 @@ setMethod(
             hessian = FALSE,
             control = list(
               maxit = maxit,
-              reltol = reltol
+              reltol = reltol,
+              fnscale = -1
             )
           )
         )
         par_g <- opt$par
         if (k %% every == 0) {
           cat("\r", "iteration:", k,
-            ", logLik:", -opt$value,
+            ", logLik:", opt$value,
             sep = " "
           )
+          if(plot == TRUE){
+            dev.off()
+            plot(head(h$breaks, length(h$density)), h$density, col = "#b2df8a", 
+                 main = "Histogram", xlab = "data", ylab = "density", type = "s", lwd = 2)
+            tmp_ph <- iph(ph(alpha_fit, S_fit), gfun = x@gfun$name, gfun_pars = par_g)
+            lines(sq, dens(tmp_ph, sq), col = "#33a02c", lwd = 2, lty = 1)
+            legend("topright", 
+                   legend = c("Data", paste("Matrix-", x@gfun$name," fit", sep ="")), 
+                   col = c("#b2df8a", "#33a02c"), 
+                   lty = c(1,1), 
+                   bty = "n", 
+                   lwd = 2, 
+                   cex = 1.2, 
+                   text.col = "black", 
+                   horiz = FALSE, 
+                   inset = c(0.05, 0.05))
+          }
         }
       }
       cat("\n", sep = "")
       x@pars$alpha <- alpha_fit
       x@pars$S <- S_fit
       x@fit <- list(
-        logLik = -opt$value,
+        logLik = opt$value,
         nobs = sum(A$weights)
       )
       x <- iph(x, gfun = x@gfun$name, gfun_pars = par_g)
@@ -379,14 +453,15 @@ setMethod(
 )
 
 data_aggregation <- function(y, w) {
+  if(length(w) == 0) w <- rep(1, length(y))
+  observations <- cbind(y, w)
+  mat <- data.frame(observations)
+  names(mat) <- c("obs", "weight")
   y <- sort(as.numeric(y))
   un_obs <- unique(y)
   if (length(w) == 0) {
     w <- rep(1, length(y))
   }
-  observations <- cbind(y, w)
-  mat <- data.frame(observations)
-  names(mat) <- c("obs", "weight")
   cum_weight <- numeric(0)
   for (i in un_obs) {
     cum_weight <- c(cum_weight, sum(mat$weight[which(mat$obs == i)]))
@@ -427,3 +502,17 @@ setMethod("logLik", "ph", function(object) {
 setMethod("coef", c(object = "ph"), function(object) {
   object@pars
 })
+
+#' LRT Method for ph Class
+#'
+#' @param x,y objects of class \linkS4class{ph}.
+#'
+#' @return LRT between the models.
+#' @export
+#' @importFrom stats pchisq
+#' 
+setMethod("LRT", c(x = "ph", y = "ph"), function(x, y) {
+  LR <- 2 * abs(logLik(y) - logLik(x))
+  degrees <- abs(attributes(logLik(y))$df - attributes(logLik(x))$df)
+  return(c(LR = LR, p.val = pchisq(LR, df = degrees, lower.tail = FALSE)))
+})
